#ifndef INSPECTOR_HPP
#define INSPECTOR_HPP

#include <QWidget>
#include <memory>

class QLineEdit;
class QTableWidget;
class QWidget;
class SceneGraphNode;
class Model;
class Light;
class Aether3D_API;
class EditorState;

/**
 * @brief The Inspector class shows selected SceneGraphNode's properties and allows manipulation.
 */
class Inspector : QWidget
{
    Q_OBJECT
public:
    Inspector( Aether3D_API* dll, EditorState* state );
    ~Inspector();
    QWidget* GetWidget();
    QLineEdit* GetNodeNameEdit();
    QTableWidget* GetTransformTable();
    QTableWidget* GetLightTable();
    void SetNodeName( const char* name );
    const char* GetNodeName() const;
    void Init();
    void Update( const SceneGraphNode* node );

public slots:
    void MeshCellClicked( int row, int col );

private:
    struct Impl;
    std::unique_ptr< Impl > m;

    void UpdateModel( const Model* model );
    void UpdateLight( const Light* light );
    Aether3D_API* const dll = nullptr;
    EditorState* const editorState = nullptr;
};

#endif // INSPECTOR_HPP
