#include "mainwindow.h"
#include <iostream>
#include <cassert>
#include <QApplication>
#include "Aether3D_Include/Aether3D.hpp"
#include "EditorState.h"

/**
  Build instructions:

  1. Copy "Include" from Aether3D_2013 into the editor's directory and name it "Aether3D_Include", it's case-sensitive.
  2. Copy libaether3d_gl32.dll or .so into "copy_to_output"
*/

std::string AbsoluteFilePath( const std::string& relativePath );

int main(int argc, char *argv[])
{
    Aether3D_API dll;

    std::string dllPath = AbsoluteFilePath( "libaether3d_gl32");
#if _MSC_VER
    std::replace( dllPath.begin(), dllPath.end(), '/', '\\');
#endif
    std::cout << "DLL path: " << dllPath << std::endl;

    const Error::Enum error = dll.LoadDLL( dllPath );

    if (error == Error::DLL_NOT_FOUND)
    {
        std::cerr << "Could not find engine DLL!" << std::endl;
        assert( !"Could not find engine DLL!" );
    }
    else if (error == Error::FACTORY_NOT_FOUND)
    {
        std::cerr << "DLL does not contain a factory method!" << std::endl;
        assert( !"No entry point in DLL!" );
    }
    else if (error == Error::DLL_VERSION_MISMATCH)
    {
        std::cerr << "Wrong DLL version" << std::endl;
        assert( !"DLL header version does not match client header version!" );
    }

    EditorState editorState;
    QApplication a(argc, argv);
    a.setKeyboardInputInterval(10);
    MainWindow w( &a, &dll, &editorState );
    w.showMaximized();
    w.SetSelectedNode( nullptr );

    return a.exec();
}
