#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <memory>
#include <QMainWindow>

class QAction;
class QTableWidgetItem;
class QTreeWidget;
class QTreeWidgetItem;
class QWidget;
class MyGLWidget;
class SceneGraphNode;
class Aether3D_API;
class EditorState;

namespace Ui {
class MainWindow;
}

/**
 * MainWindow contains scene hierarchy view, Inspector and 3D view of the Scene (MyGLWidget)
 **/
class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow( QApplication* app, Aether3D_API* dll, EditorState* state, QWidget *parent = 0 );
    ~MainWindow();
    void RemoveNode( SceneGraphNode* node );
    void SetSelectedNode( SceneGraphNode* model );
    void UpdateInspector();
    void NodeNameChanged();
    void InspectorTransformChanged( QTableWidgetItem* item );
    void InspectorLightChanged( QTableWidgetItem* item );
    void UpdateHierarchy();
    void Save();

protected:
    void closeEvent( QCloseEvent *event );

private slots:
    void OpenScene();
    void OpenRecentScene(QAction* action);
    void OpenModel();
    void SaveScene();
    void CreatePointLight();
    void CreateDirectionalLight();
    void CreateEmptyNode();
    void CreateCamera();

private:
    struct Impl;
    std::unique_ptr< Impl > m;

    void InitMenu();
    void keyPressEvent( QKeyEvent* event );
    void SelectTreeItem( QTreeWidgetItem* item );

    MyGLWidget* sceneWidget = nullptr;
    QTreeWidget* sceneTree = nullptr;
    EditorState* const editorState = nullptr;
};

#endif // MAINWINDOW_H
