#ifndef MYGLWIDGET_HPP
#define MYGLWIDGET_HPP

#include <map>
#include <string>
#include <QtOpenGL/QGLWidget>
#include <QTimer>
#include <QDesktopWidget>
#include "Aether3D_Include/Aether3D.hpp"

class Camera;
class Material;
class MainWindow;
class Model;
class Light;
class Renderer;
class Scene;
class Texture;
class Aether3D_API;
class EditorState;

struct GizmoMaterials
{
    Texture* xTexture = nullptr;
    Texture* yTexture = nullptr;
    Texture* zTexture = nullptr;

    Material* xAxis = nullptr;
    Material* xAxisHighlight = nullptr;
    Material* yAxis = nullptr;
    Material* yAxisHighlight = nullptr;
    Material* zAxis = nullptr;
    Material* zAxisHighlight = nullptr;
};

class MyGLWidget : public QGLWidget
{
    Q_OBJECT
public:
    explicit MyGLWidget(const QGLFormat& format, QApplication* app, Aether3D_API* dll, MainWindow* mainWindow, EditorState* state, QWidget * parent = 0);
    explicit MyGLWidget(QWidget *parent = 0);
    Scene* GetScene() { return scene; }
    Camera* GetEditorCamera() { return editorCamera; }
    Model* AddModel( const char* path );
    Light* AddLight( Light::Type type );
    void AddEmpty();
    void AddCamera();
    void PreSave();
    void PostSave();
    void RemoveNode( SceneGraphNode* node );
    void CenterSelected();
    void SelectionPositionChanged( const Vec3& newPosition );
    bool IsEditorWidget( const SceneGraphNode* node ) const;
    void SetSelectedNodePosition( const Vec3& newPosition );
    SceneGraphNode* RootNode() { return editorCamera->Parent(); }
    Texture* CameraRenderTexture() { return cameraRenderTexture; }
    void DeinitRenderer();
    void SelectionChanged();

protected:
    void initializeGL();
    void paintGL();
    void resizeGL( int width, int height );
    void keyPressEvent( QKeyEvent* event );
    void keyReleaseEvent( QKeyEvent* event );
    void wheelEvent(QWheelEvent *event);
    void mousePressEvent( QMouseEvent* event );
    void mouseReleaseEvent( QMouseEvent* event );
    void updateGL();
    bool eventFilter(QObject *obj, QEvent *event);

signals:

public slots:
    void updateCamera();

private:
    enum class GizmoAxis { None, X, Y, Z };
    void DrawCameraSprites();
    void DrawLightSprites();
    void DrawSelectionHighlight();
    bool HandleDrag( const QMouseEvent* mouseEvent, float deltaX, float deltaY );
    void HandleCursorHighlight();
    bool IsLightWidget( SceneGraphNode* node ) const;
    void InitGizmoMaterials();
    void TryToSelectSprite( int x, int y );

    Renderer* renderer = nullptr;
    Camera* editorCamera = nullptr;
    Scene* scene = nullptr;
    Model* sphere = nullptr;
    Model* cursorTranslate = nullptr;
    Material* unlit = nullptr;
    std::map< std::string, Material* > meshNameToMaterial;
    GizmoMaterials gizmoMaterials;
    Texture* checker = nullptr;
    Texture* lightTex = nullptr;
    Texture* cameraTex = nullptr;
    Texture* cameraRenderTexture = nullptr; // Cameras cannot render into backbuffer, they confuse the editor.
    int buttonDownMousePosition[ 2 ];
    int lastMousePosition[ 2 ];
    bool grabMouse = false;
    bool panMouse = false;

    Aether3D_API* const dll = nullptr;
    QApplication* const app = nullptr;
    MainWindow* const mainWindow = nullptr;
    EditorState* const editorState = nullptr;

    QTimer myTimer;
    int deltaTimeMs = 0;
    int lastTimeMs = 0;
    Vec3 cameraMoveDir; // Keypress events set these to allow smooth movement.
    GizmoAxis dragAxis = GizmoAxis::None;
    QDesktopWidget desktop;
};

#endif // MYGLWIDGET_HPP
